<?php

$ABS_PATH = dirname(__FILE__);
include_once $ABS_PATH.'/settings.php';
include_once $ABS_PATH.'/db.inc.php';

/* Securely start a PHP session */
function sec_session_start() {
	/*
    $session_name = 'sec_session_id';   // Set a custom session name
    $secure = SECURE;
    // This stops JavaScript being able to access the session id.
    $httponly = true;
    // Forces sessions to only use cookies.
    if (ini_set('session.use_only_cookies', 1) === FALSE) {
        header("Location: ../error.php?err=Could not initiate a safe session (ini_set)");
        exit();
    }
    // Gets current cookies params.
    $cookieParams = session_get_cookie_params();
    session_set_cookie_params($cookieParams["lifetime"],
        $cookieParams["path"], 
        $cookieParams["domain"], 
        $secure,
        $httponly);
    // Sets the session name to the one set above.
    session_name($session_name);
    session_start();            // Start the PHP session 
    session_regenerate_id();    // regenerated the session, delete the old one.
	*/
	session_start();
}

/* Login Function */
function login($email, $password, $dbConn) {
	global $cfg;
	
    // Using prepared statements means that SQL injection is not possible. 
    if ($stmt = $dbConn->prepare("SELECT * 
        FROM ".$cfg['DB_TABLE_USERS']."
        WHERE email = :email
		LIMIT 1")) {
		//$stmt->bindParam(':email', $email); // Bind "$email" to parameter.
		$stmt->bindValue(':email', $email); // Bind "$email" to parameter.
        $res = $stmt->execute();    // Execute the prepared query.

		$foundedLogins=0;
		if($stmt->rowCount() > 0) {
			// get variables from result.
			$stmt->bindColumn('id', $user_id);
			$stmt->bindColumn('access_level', $access_level);
			$stmt->bindColumn('email', $email);
			$stmt->bindColumn('password', $db_password);
			$stmt->bindColumn('enabled', $enabled);
			$stmt->bindColumn('registration_hash', $registration_hash);
			$stmt->bindColumn('salt', $salt);
			$foundedLogins = $stmt->fetch(PDO::FETCH_BOUND);
		}

        if ($foundedLogins == 1) {
		
			if($enabled != 1) {
				echo $cfg['MSG_LOGIN_NOT_ENABLED'];
			}
			elseif($registration_hash != "confirmed") {
				echo $cfg['MSG_EMAIL_NOT_CONFIRMED'];
			}
			else {
				// hash the password with the unique salt.
				$password = hash('sha512', $password . $salt);
				// If the user exists we check if the account is locked
				// from too many login attempts 
				
				if (checkbrute($email, $dbConn) == true) {
					// Account is locked 
					// Send an email to user saying their account is locked
					echo $cfg['MSG_LOGIN_FAILED'];
					return false;
				} else {
					// Check if the password in the database matches
					// the password the user submitted.
					if ($db_password == $password) {
						// Password is correct!
						// Get the user-agent string of the user.
						$user_browser = $_SERVER['HTTP_USER_AGENT'];
						// XSS protection as we might print this value
						$user_id = preg_replace("/[^0-9]+/", "", $user_id);
						$_SESSION['user_id'] = $user_id;
						$_SESSION['access_level'] = $access_level;
						// XSS protection as we might print this value
						//$username = preg_replace("/[^a-zA-Z0-9_\-]+/", "", $username);
						//$_SESSION['username'] = $username;
						$_SESSION['login_string'] = hash('sha512', $password . $user_browser);
						setCSRFToken(true);

						// Login successful.
						return true;
					} else {
						// Password is not correct
						// We record this attempt in the database
						$now = time();
						$dbConn->query("INSERT INTO ".$cfg['DB_TABLE_LOGIN_ATTEMPTS']." (email, time)
										VALUES ('$email', '$now')");
										
						echo $cfg['MSG_LOGIN_FAILED'];
						return false;
					}
				}
			}
        } else {
            // No user exists.
			echo $cfg['MSG_LOGIN_FAILED'];
            return false;
        }
    }
}

/* Process Login Function */
function process_login($dbConn) {
	global $cfg;
	
	sec_session_start(); // Our custom secure way of starting a PHP session.

	if (isset($_POST['email'], $_POST['passwordHashed'])) {
		$email = $_POST['email'];
		$password = $_POST['passwordHashed']; // The hashed password.
	 
		if (login($email, $password, $dbConn) == true) {
			// Login success 
			#header('Location: ../index.php');
			echo 'OK';
		} else {
			// Login failed 
			echo '';
		}
	} else {
		// The correct POST variables were not sent to this page. 
		echo $cfg['MSG_INVALID_REQUEST'];
	}
}

/* Process PWD Change Function */
function process_pwdchange($dbConn) {
	global $cfg;
	
	sec_session_start(); // Our custom secure way of starting a PHP session.
	$error_msg = "";
	
	if(chk_is_logged_in() == true || isset($_POST['pwdchangehash'])) {
		if ((isset($_POST['new_passwordHashed'], $_POST['new_password_confHashed']) || isset($_POST['new_password_resHashed'], $_POST['new_password_res_confHashed'])) && (isset($_POST['old_passwordHashed']) || isset($_POST['pwdchangehash']))) {
			
			$old_password = $_POST['old_passwordHashed']; // The hashed password.
			$lostpwd_hash = $_POST['pwdchangehash']; // The hashed password.
			$new_password = $_POST['new_passwordHashed']; // The hashed password.
			$new_password_conf = $_POST['new_password_confHashed']; // The hashed password.
				
			if(isset($_POST['new_password_resHashed'])) {
				$new_password = $_POST['new_password_resHashed']; // The hashed password.
				$new_password_conf = $_POST['new_password_res_confHashed']; // The hashed password.
			}

			if ((strlen($old_password) != 128 && strlen($lostpwd_hash) != 128) || strlen($new_password) != 128 || strlen($new_password_conf) != 128) {
				// The hashed pwd should be 128 characters long.
				// If it's not, something really odd has happened
				$error_msg = $cfg['MSG_INVALID_PASSWORD'];
			}

			// Check if the new password and the password confirmation are equal
			if($new_password != $new_password_conf) {
				$error_msg = $cfg['MSG_CHECK_INPUT'];
			}
			
			// check if the given old pwd is correct
			if(isset($lostpwd_hash)) {
				$usr = get_user_data_by_lostpwd_hash($lostpwd_hash);
			}
			else {
				$usr = get_user_data($_SESSION['user_id']);
			}
			$user_id = $usr['id'];
			
			$dbpwd = $usr['password'];
			$dblostpwd_hash = $usr['lostpwd_hash'];
			$old_password = hash('sha512', $old_password . $usr['salt']);
			if($old_password != $dbpwd && $lostpwd_hash != $dblostpwd_hash) {
				$error_msg = $cfg['MSG_CHECK_INPUT'];
			}
			
			if (empty($error_msg)) {
				$random_salt = hash('sha512', uniqid(generateRandom(16), TRUE));
				$new_password = hash('sha512', $new_password . $random_salt);

				$res = sqlSetNewPwd($user_id, $new_password, $random_salt);
				echo $res;
			}
			else {
				echo $error_msg;
			}
			
		} else {
			// The correct POST variables were not sent to this page. 
			echo $cfg['MSG_INVALID_REQUEST'];
		}
	}
}

/* Login Function */
function logout() {
	sec_session_start();
 
	// Unset all session values 
	$_SESSION = array();
	 
	// get session parameters 
	$params = session_get_cookie_params();
	 
	// Delete the actual cookie. 
	setcookie(session_name(),
			'', time() - 42000, 
			$params["path"], 
			$params["domain"], 
			$params["secure"], 
			$params["httponly"]);
	 
	// Destroy session 
	session_destroy();
}

/* The Brute Force Function */
function checkbrute($email, $dbConn) {
	global $cfg;
	
    // Get timestamp of current time 
    $now = time();
 
    // All login attempts are counted from the past 2 hours. 
    $valid_attempts = $now - (2 * 60 * 60);
 
    if ($stmt = $dbConn->prepare("SELECT time 
                             FROM ".$cfg['DB_TABLE_LOGIN_ATTEMPTS']."
                             WHERE email = :email 
                             AND time > '$valid_attempts'")) {
        $stmt->bindParam('email', $email);
 
        // Execute the prepared query. 
        $stmt->execute();

		$failedLogins=0;
		while ( $row = $stmt->fetch( PDO::FETCH_ASSOC ) ){
			$failedLogins++;
		}
		
        // If there have been more than 5 failed logins 
        if ($failedLogins > $cfg['ALLOW_FAILED_LOGINS']) {
            return true;
        } else {
            return false;
        }
    }
}

/* check logged in status */
function login_check($dbConn) {
	global $cfg;
	
    // Check if all session variables are set 
    if (isset($_SESSION['user_id'], $_SESSION['login_string'])) {
 
        $user_id = $_SESSION['user_id'];
        $login_string = $_SESSION['login_string'];
 
        // Get the user-agent string of the user.
        $user_browser = $_SERVER['HTTP_USER_AGENT'];
 
        if ($stmt = $dbConn->prepare("SELECT email, password
                                      FROM ".$cfg['DB_TABLE_USERS']." 
                                      WHERE id = :id
										AND enabled = 1
										AND registration_hash = 'confirmed'
									  LIMIT 1")) {
            // Bind "$user_id" to parameter. 
            $stmt->bindParam(':id', $user_id);
            $stmt->execute();   // Execute the prepared query.
			$stmt->bindColumn('email', $email);
			$stmt->bindColumn('password', $password);
			$stmt->fetch(PDO::FETCH_BOUND);

            if ($email) {
                // If the user exists get variables from result.
                $login_check = hash('sha512', $password . $user_browser);
                if ($login_check == $login_string) {
                    // Logged In!!!!
                    return true;
                } else {
                    // Not logged in 
                    return false;
                }
            } else {
                // Not logged in 
                return false;
            }
        } else {
            // Not logged in 
            return false;
        }
    } else {
        // Not logged in 
        return false;
    }
}


/* check if password has been expired */
function getPasswordExiredDays($dbConn) {
	global $cfg;

    $user_id = $_SESSION['user_id'];
    $stmt = $dbConn->prepare("SELECT CASE
												WHEN last_pwd_change IS NULL THEN 99999
												ELSE DATEDIFF(day, last_pwd_change, GETDATE())
												END as last_pwd_change_in_days
                              FROM ".$cfg['DB_TABLE_USERS']." 
                              WHERE id = :id
									and enabled = 1
									AND registration_hash = 'confirmed'
							   LIMIT 1");
    // Bind "$user_id" to parameter. 
    $stmt->bindParam(':id', $user_id);
    $stmt->execute();   // Execute the prepared query.
	$stmt->bindColumn('last_pwd_change_in_days', $last_pwd_change_in_days);
	$stmt->fetch(PDO::FETCH_BOUND);

	// Check if password has been expired
	if($last_pwd_change_in_days == '' OR ($last_pwd_change_in_days >= $cfg['CHANGE_PWD_AFTER_X_DAYS'])) {
		return  $last_pwd_change_in_days;
	}
    return 0;
}

/* Sanitize URL from PHP_SELF */
function esc_url($url) {
 
    if ('' == $url) {
        return $url;
    }
 
    $url = preg_replace('|[^a-z0-9-~+_.?#=!&;,/:%@$\|*\'()\\x80-\\xff]|i', '', $url);
 
    $strip = array('%0d', '%0a', '%0D', '%0A');
    $url = (string) $url;
 
    $count = 1;
    while ($count) {
        $url = str_replace($strip, '', $url, $count);
    }
 
    $url = str_replace(';//', '://', $url);
 
    $url = htmlentities($url);
 
    $url = str_replace('&amp;', '&#038;', $url);
    $url = str_replace("'", '&#039;', $url);
 
    if ($url[0] !== '/') {
        // We're only interested in relative links from $_SERVER['PHP_SELF']
        return '';
    } else {
        return $url;
    }
}

function generateRandom($length) {
   $validCharacters = 'abcdefghijklmnopqrstuvwxyz0123456789';
   $myKeeper = '';
   for ($n = 1; $n < $length; $n++) {
      $whichCharacter = rand(0, strlen($validCharacters)-1);
      $myKeeper .= $validCharacters{$whichCharacter};
   }
   return $myKeeper;
}

// get session_data
function get_session_data($key) {
	//$key = preg_replace("/[^a-zA-Z0-9_\-]+/", "", $key);
	return $_SESSION[$key];
}

function setCSRFToken($setAnyway = true) {
	
	if($setAnyway == true) {
		$_SESSION['CSRF_TOKEN'] = hash('sha512', generateRandom(50) . microtime());
	}
	else {
		if(!isset($_SESSION['CSRF_TOKEN'])) {
			$_SESSION['CSRF_TOKEN'] = hash('sha512', generateRandom(50) . microtime());
		}
	}
}

function checkCSRFToken() {
	if(count($_POST)>0
		AND ($_POST['CSRF_TOKEN'] != get_session_data('CSRF_TOKEN')))
		return false;
	
	return true;
}

function is_logged_in() {
	if(!isset($_SESSION['login_string'])) {
		exit;
	}
}

function chk_is_logged_in() {
	if(isset($_SESSION['login_string'])) {
		return true;
	}
	return false;
}

function get_user_access_level() {
	return $_SESSION['access_level'];
}

function get_user_data($id) {
	global $dbConn, $cfg;

    $user_id = $_SESSION['user_id'];
    $stmt = $dbConn->prepare("SELECT * from ".$cfg['DB_TABLE_USERS']." WHERE id = :id LIMIT 1");
	$stmt->bindParam(':id', $id);
    $stmt->execute();   // Execute the prepared query.
	$row = $stmt->fetch();

	return $row;
}

function get_user_data_by_lostpwd_hash($lostpwd_hash) {
	global $dbConn, $cfg;

	if(isset($lostpwd_hash)) {
		$stmt = $dbConn->prepare("SELECT * from ".$cfg['DB_TABLE_USERS']." WHERE lostpwd_hash = :lostpwd_hash LIMIT 1");
		$stmt->bindParam(':lostpwd_hash', $lostpwd_hash);
		$stmt->execute();   // Execute the prepared query.
		$row = $stmt->fetch();

		return $row;
	}
	
	return "";
}

function sqlGetUsers() {
	global $dbConn, $cfg;

    $stmt = $dbConn->prepare("SELECT * from ".$cfg['DB_TABLE_USERS']);
    $stmt->execute();   // Execute the prepared query.

	while ( $row = $stmt->fetch( PDO::FETCH_ASSOC ) ){
		$res[] = $row;
	}

	return $res;
}

function sqlSetUserStatus($user_id, $status) {
	global $dbConn, $cfg;

	// Activate/Deactivate User
    if ($stmt = $dbConn->prepare("UPDATE ".$cfg['DB_TABLE_USERS']."
								  SET enabled = :enabled
								  WHERE id = :id")) {
		$stmt->bindParam(':id', $user_id);
		$stmt->bindParam(':enabled', $status);

        // Execute the prepared query.
        if (! $stmt->execute()) {
			return $cfg['SQL_ERROR'];
        }
		else {
			return "OK";
		}
    }
	else {
		return $cfg['SQL_ERROR'];
	}
}

function sqlSetNewPwd($user_id, $password, $salt) {
	global $dbConn, $cfg;
	
	// Activate/Deactivate User
    if ($stmt = $dbConn->prepare("UPDATE ".$cfg['DB_TABLE_USERS']."
								  SET password = :password, salt = :salt, lostpwd_hash = NULL
								  WHERE id = :id")) {
		$stmt->bindParam(':id', $user_id);
		$stmt->bindParam(':password', $password);
		$stmt->bindParam(':salt', $salt);

        // Execute the prepared query.
        if (! $stmt->execute()) {
			return $cfg['SQL_ERROR'];
        }
		else {
			return "OK";
		}
    }
	else {
		return $cfg['SQL_ERROR'];
	}
}

function sqlSetLostPwdHash($email, $lostpwd_hash) {
	global $dbConn, $cfg;
	
	// Activate/Deactivate User
    if ($stmt = $dbConn->prepare("UPDATE ".$cfg['DB_TABLE_USERS']."
								  SET lostpwd_hash = :lostpwd_hash
								  WHERE email = :email")) {
		$stmt->bindParam(':email', $email);
		$stmt->bindParam(':lostpwd_hash', $lostpwd_hash);

        // Execute the prepared query.
        if (! $stmt->execute()) {
			return $cfg['SQL_ERROR'];
        }
		else {
			return "OK";
		}
    }
	else {
		return $cfg['SQL_ERROR'];
	}
}

?>