<?php
$ABS_PATH = dirname(__FILE__);

include_once $ABS_PATH.'/settings.php';
include_once $ABS_PATH.'/login.inc.php';
include_once $ABS_PATH.'/mail.inc.php';

 /* Process Login Function */
function process_registration($dbConn) {
	global $cfg;

	$error_msg = "";

	$_POST['CSRF_TOKEN'] = "test";

	if (isset($_POST['email'], $_POST['passwordHashed'], $_POST['CSRF_TOKEN'])) {

		// Sanitize and validate the data passed in
		$first_name = filter_input(INPUT_POST, 'first_name', FILTER_SANITIZE_STRING);
		$last_name = filter_input(INPUT_POST, 'last_name', FILTER_SANITIZE_STRING);
		$tel = filter_input(INPUT_POST, 'tel', FILTER_SANITIZE_STRING);
		$email = filter_input(INPUT_POST, 'email', FILTER_SANITIZE_EMAIL);
		
		$email = filter_var($email, FILTER_VALIDATE_EMAIL);
		if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
			// Not a valid email
			$error_msg = $cfg['MSG_INVALID_EMAIL'];
		}
	 
		$password = filter_input(INPUT_POST, 'passwordHashed', FILTER_SANITIZE_STRING);
		if (strlen($password) != 128) {
			// The hashed pwd should be 128 characters long.
			// If it's not, something really odd has happened
			$error_msg = $cfg['MSG_INVALID_PASSWORD'];
		}

		// email validity and password validity have been checked client side.
		// This should should be adequate as nobody gains any advantage from
		// breaking these rules.
		$prep_stmt = "SELECT id FROM ".$cfg['DB_TABLE_USERS']." WHERE email = :email LIMIT 1";
		$stmt = $dbConn->prepare($prep_stmt);
		$stmt->bindParam(':email', $email);
		$stmt->execute();

		$foundedLogins=0;
		if($stmt->ColumnCount() > 0) {
			$foundedLogins = $stmt->fetch(PDO::FETCH_BOUND);
		}
		
		if ($foundedLogins > 0) {
			$stmt->bindColumn('id', $id);
			$stmt->fetch(PDO::FETCH_BOUND);
	 
			if ($stmt->ColumnCount() > 0) {
				// A user with this email address already exists
				$error_msg = $cfg['MSG_EMAIL_EXISTS'];
			}
		}

		if (empty($error_msg)) {
			// Create a random salt
			//$random_salt = hash('sha512', uniqid(openssl_random_pseudo_bytes(16), TRUE));
			$random_salt = hash('sha512', uniqid(generateRandom(16), TRUE));
	 
			// Create salted password 
			$password = hash('sha512', $password . $random_salt);
			
			$registration_hash = hash('sha512', uniqid(generateRandom(16), TRUE));
	 
			// Insert the new user into the database 
			if ($insert_stmt = $dbConn->prepare("INSERT INTO ".$cfg['DB_TABLE_USERS']." (enabled, email, password, salt, first_name, last_name, tel, registration_hash)
															  VALUES (0, :email, :password, :salt, :first_name, :last_name, :tel, :registration_hash)")) {
				$insert_stmt->bindParam(':email', $email);
				$insert_stmt->bindParam(':password', $password);
				$insert_stmt->bindParam(':salt', $random_salt);
				$insert_stmt->bindParam(':first_name', $first_name);
				$insert_stmt->bindParam(':last_name', $last_name);
				$insert_stmt->bindParam(':tel', $tel);
				$insert_stmt->bindParam(':registration_hash', $registration_hash);

				// Execute the prepared query.
				if (! $insert_stmt->execute()) {
					echo $cfg['MSG_SQL_ERROR'];
				}
				else {
					$message_admin = $cfg['MAIL_REGISTRATION_ADMIN_MESSAGE'];
					$message_admin = str_replace("%email%", $email, $message_admin);
					$message_admin = str_replace("%first_name%", $first_name, $message_admin);
					$message_admin = str_replace("%last_name%", $last_name, $message_admin);
					$message_admin = str_replace("%tel%", $tel, $message_admin);
					sendMail($cfg['MAIL_ADMIN'], $cfg['MAIL_REGISTRATION_SUBJECT'], $message_admin);
					
					$message_user = $cfg['MAIL_REGISTRATION_USER_MESSAGE'];
					$message_user = str_replace("%registration_hash%", $registration_hash, $message_user);
					$message_user = str_replace("%USER_NAME%", $email, $message_user);
					sendMail($email, $cfg['MAIL_REGISTRATION_SUBJECT'], $message_user);
					
					echo $cfg['MSG_REGISTRATION_OK'];
				}
			}
		}
		else {
			echo $error_msg;
		}
	}
	else {
		// The correct POST variables were not sent to this page. 
		echo $cfg['MSG_INVALID_REQUEST'];
	}
}

function process_mail_confirm($dbConn) {
	global $cfg;

	$error_msg = "";

	if (isset($_REQUEST['hash'])) {
		// Sanitize and validate the data passed in
		$hash = filter_input(INPUT_GET, 'hash', FILTER_SANITIZE_STRING);
		if (strlen($hash) != 128) {
			// The hashed pwd should be 128 characters long.
			// If it's not, something really odd has happened
			$error_msg = $cfg['MSG_INVALID_HASH'];
		}

		if (empty($error_msg)) {
			$MAX_EMAIL_CONFIRM_HOURS = date("Y-m-d H:i:s", time() - ($cfg['MAX_EMAIL_CONFIRM_HOURS'] * 60 * 60));

			$prep_stmt = "SELECT id FROM ".$cfg['DB_TABLE_USERS']." WHERE registration_hash = :registration_hash AND registration_date >= :MAX_EMAIL_CONFIRM_HOURS LIMIT 1";
			$stmt = $dbConn->prepare($prep_stmt);
			$stmt->bindParam(':registration_hash', $hash);
			$stmt->bindParam(':MAX_EMAIL_CONFIRM_HOURS', $MAX_EMAIL_CONFIRM_HOURS);
			$stmt->execute();

			$foundedLogins=0;
			if($stmt->rowCount() > 0) {
				$stmt->bindColumn('id', $id);
				$foundedLogins = $stmt->fetch(PDO::FETCH_BOUND);
			}

			if ($foundedLogins == 1) {

				// Hash found
				if ($stmt->ColumnCount() > 0) {
					
					$prep_stmt = "UPDATE ".$cfg['DB_TABLE_USERS']." SET registration_hash = 'confirmed' WHERE id = :id AND registration_hash = :registration_hash AND registration_date >= :MAX_EMAIL_CONFIRM_HOURS LIMIT 1";
					$stmt = $dbConn->prepare($prep_stmt);
					$stmt->bindParam(':id', $id);
					$stmt->bindParam(':registration_hash', $hash);
					$stmt->bindParam(':MAX_EMAIL_CONFIRM_HOURS', $MAX_EMAIL_CONFIRM_HOURS);
					
					// Execute the prepared query.
					if (! $stmt->execute()) {
						echo $cfg['MSG_INVALID_HASH'];
					}
					else {
						echo $cfg['MSG_EMAIL_CONFIRMED'];
					}
				}
			}
			else {
				echo $cfg['MSG_INVALID_HASH'];
			}
		}
		else {
			echo $error_msg;
		}
	
	}
	else {
		// The correct POST variables were not sent to this page. 
		echo $cfg['MSG_INVALID_REQUEST'];
	}
	
}

?>